@@ -15,6 +15,7 @@
 #include "configuration.h"
 #include "x-server-local.h"
 #include "unity-system-compositor.h"
+#include "wayland-session.h"
 #include "plymouth.h"
 #include "vt.h"
 
@@ -110,7 +111,7 @@ seat_xlocal_start (Seat *seat)
             g_key_file_free (keys);
         }
 
-        g_signal_connect (s->priv->xdmcp_x_server, "stopped", G_CALLBACK (xdmcp_x_server_stopped_cb), seat);
+        g_signal_connect (s->priv->xdmcp_x_server, DISPLAY_SERVER_SIGNAL_STOPPED, G_CALLBACK (xdmcp_x_server_stopped_cb), seat);
         return display_server_start (DISPLAY_SERVER (s->priv->xdmcp_x_server));
     }
 
@@ -150,8 +151,8 @@ get_vt (Seat *seat, DisplayServer *display_server)
         if (active_vt >= vt_get_min ())
         {
             vt = active_vt;
-            g_signal_connect (display_server, "ready", G_CALLBACK (display_server_ready_cb), seat);
-            g_signal_connect (display_server, "stopped", G_CALLBACK (display_server_transition_plymouth_cb), seat);
+            g_signal_connect (display_server, DISPLAY_SERVER_SIGNAL_READY, G_CALLBACK (display_server_ready_cb), seat);
+            g_signal_connect (display_server, DISPLAY_SERVER_SIGNAL_STOPPED, G_CALLBACK (display_server_transition_plymouth_cb), seat);
             plymouth_deactivate ();
         }
         else
@@ -197,8 +198,6 @@ create_x_server (Seat *seat)
         x_server_local_set_layout (x_server, layout);
 
     x_server_local_set_xdg_seat (x_server, seat_get_name (seat));
-    if (strcmp (seat_get_name (seat), "seat0") != 0)
-        x_server_local_set_sharevts (x_server, TRUE);
 
     config_file = seat_get_string_property (seat, "xserver-config");
     if (config_file)
@@ -247,6 +246,21 @@ create_unity_system_compositor (Seat *seat)
     return DISPLAY_SERVER (compositor);
 }
 
+static DisplayServer *
+create_wayland_session (Seat *seat)
+{
+    WaylandSession *session;
+    gint vt;
+
+    session = wayland_session_new ();
+
+    vt = get_vt (seat, DISPLAY_SERVER (session));
+    if (vt >= 0)
+        wayland_session_set_vt (session, vt);
+
+    return DISPLAY_SERVER (session);
+}
+
 static DisplayServer *
 seat_xlocal_create_display_server (Seat *seat, Session *session)
 {
@@ -257,6 +271,8 @@ seat_xlocal_create_display_server (Seat *seat, Session *session)
         return DISPLAY_SERVER (create_x_server (seat));
     else if (strcmp (session_type, "mir") == 0)
         return create_unity_system_compositor (seat);
+    else if (strcmp (session_type, "wayland") == 0)
+        return create_wayland_session (seat);
     else if (strcmp (session_type, "mir-container") == 0)
     {
         DisplayServer *compositor;
