@@ -74,8 +74,65 @@ struct XServerLocalPrivate
 
 G_DEFINE_TYPE (XServerLocal, x_server_local, X_SERVER_TYPE);
 
+static gchar *version = NULL;
+static guint version_major = 0, version_minor = 0;
 static GList *display_numbers = NULL;
 
+#define XORG_VERSION_PREFIX "X.Org X Server "
+
+static gchar *
+find_version (const gchar *line)
+{
+    if (!g_str_has_prefix (line, XORG_VERSION_PREFIX))
+        return NULL;
+
+    return g_strdup (line + strlen (XORG_VERSION_PREFIX));
+}
+
+const gchar *
+x_server_local_get_version (void)
+{
+    gchar *stderr_text;
+    gint exit_status;
+    gchar **tokens;
+    guint n_tokens;
+
+    if (version)
+        return version;
+
+    if (!g_spawn_command_line_sync ("X -version", NULL, &stderr_text, &exit_status, NULL))
+        return NULL;
+    if (exit_status == EXIT_SUCCESS)
+    {
+        gchar **lines;
+        int i;
+
+        lines = g_strsplit (stderr_text, "\n", -1);
+        for (i = 0; lines[i] && !version; i++)
+            version = find_version (lines[i]);
+        g_strfreev (lines);
+    }
+    g_free (stderr_text);
+
+    tokens = g_strsplit (version, ".", 3);
+    n_tokens = g_strv_length (tokens);
+    version_major = n_tokens > 0 ? atoi (tokens[0]) : 0;
+    version_minor = n_tokens > 1 ? atoi (tokens[1]) : 0;
+    g_strfreev (tokens);
+
+    return version;
+}
+
+gint
+x_server_local_version_compare (guint major, guint minor)
+{
+    x_server_local_get_version ();
+    if (major == version_major)
+        return version_minor - minor;
+    else
+        return version_major - major;
+}
+
 static gboolean
 display_number_in_use (guint display_number)
 {
@@ -417,7 +474,7 @@ static gboolean
 x_server_local_start (DisplayServer *display_server)
 {
     XServerLocal *server = X_SERVER_LOCAL (display_server);
-    gboolean result;
+    gboolean result, backup_logs;
     gchar *filename, *dir, *log_file, *absolute_command;
     GString *command;
 
@@ -436,7 +493,8 @@ x_server_local_start (DisplayServer *display_server)
     filename = g_strdup_printf ("%s.log", display_server_get_name (display_server));
     dir = config_get_string (config_get_instance (), "LightDM", "log-directory");
     log_file = g_build_filename (dir, filename, NULL);
-    process_set_log_file (server->priv->x_server_process, log_file, TRUE);
+    backup_logs = config_get_boolean (config_get_instance (), "LightDM", "backup-logs");
+    process_set_log_file (server->priv->x_server_process, log_file, TRUE, backup_logs ? LOG_MODE_BACKUP_AND_TRUNCATE : LOG_MODE_APPEND);
     l_debug (display_server, "Logging to %s", log_file);
     g_free (log_file);
     g_free (filename);
@@ -483,7 +541,12 @@ x_server_local_start (DisplayServer *display_server)
         if (server->priv->xdmcp_key)
             g_string_append_printf (command, " -cookie %s", server->priv->xdmcp_key);
     }
-    else if (!server->priv->allow_tcp)
+    else if (server->priv->allow_tcp)
+    {
+        if (x_server_local_version_compare (1, 17) >= 0)
+            g_string_append (command, " -listen tcp");
+    }
+    else
         g_string_append (command, " -nolisten tcp");
 
     if (server->priv->vt >= 0)
